@@ -42,7 +42,7 @@ uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::Is64KBPageSuitable()
        Surf.Flags.Gpu.CameraCapture ||
        Surf.Flags.Info.KernelModeMapped ||
        (Surf.Flags.Gpu.S3d && !Surf.Flags.Gpu.S3dDx &&
-        !pGmmGlobalContext->GetSkuTable().FtrDisplayEngineS3d)
+        !GetGmmLibContext()->GetSkuTable().FtrDisplayEngineS3d)
 #if(LHDM)
        || (Surf.Flags.Info.AllowVirtualPadding &&
            ExistingSysMem.hParentAllocation)
@@ -52,23 +52,24 @@ uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::Is64KBPageSuitable()
         Ignore64KBPadding = true;
     }
 
-    if(pGmmGlobalContext->GetSkuTable().FtrLocalMemory)
-    {
+    if(GetGmmLibContext()->GetSkuTable().FtrLocalMemory)
+     {
         Ignore64KBPadding |= (Surf.Flags.Info.NonLocalOnly || (Surf.Flags.Info.Shared && !Surf.Flags.Info.NotLockable));
-        Ignore64KBPadding |= (pGmmGlobalContext->GetSkuTable().FtrLocalMemoryAllows4KB && Surf.Flags.Info.NoOptimizationPadding);
+        Ignore64KBPadding |= (GetGmmLibContext()->GetSkuTable().FtrLocalMemoryAllows4KB && Surf.Flags.Info.NoOptimizationPadding);
+	Ignore64KBPadding |= ((GetGmmLibContext()->GetSkuTable().FtrLocalMemoryAllows4KB) && (((Size * (100 + (GMM_GFX_SIZE_T)GetGmmLibContext()->GetAllowedPaddingFor64KbPagesPercentage())) / 100) < GFX_ALIGN(Size, GMM_KBYTE(64)))); 
     }
     else
     {
         // The final padded size cannot be larger then a set percentage of the original size
         if((Surf.Flags.Info.NoOptimizationPadding && !GFX_IS_ALIGNED(Size, GMM_KBYTE(64))) /*Surface is not 64kb aligned*/ ||
-           (!Surf.Flags.Info.NoOptimizationPadding && (((Size * (100 + pGmmGlobalContext->GetAllowedPaddingFor64KbPagesPercentage())) / 100) < GFX_ALIGN(Size, GMM_KBYTE(64)))) /*10% padding TBC */)
+           (!Surf.Flags.Info.NoOptimizationPadding && (((Size * (100 + GetGmmLibContext()->GetAllowedPaddingFor64KbPagesPercentage())) / 100) < GFX_ALIGN(Size, GMM_KBYTE(64)))) /*10% padding TBC */)
         {
             Ignore64KBPadding |= true;
         }
     }
 
     // If 64KB paging is enabled pad out the resource to 64KB alignment
-    if(pGmmGlobalContext->GetSkuTable().FtrWddm2_1_64kbPages &&
+    if(GetGmmLibContext()->GetSkuTable().FtrWddm2_1_64kbPages &&
        // Ignore the padding for the above VirtualPadding or ESM cases
        (!Ignore64KBPadding) &&
        // Resource must be 64KB aligned
@@ -98,14 +99,9 @@ uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::Is64KBPageSuitable()
 GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::Create(GMM_RESCREATE_PARAMS &CreateParams)
 {
     GMM_STATUS Status = GMM_ERROR;
-
-#if defined(__GMM_KMD__)
-    ClientType = GMM_KMD_VISTA;
-#else
-    GET_GMM_CLIENT_TYPE(pClientContext, ClientType);
-#endif
-
-    Status = Create(*pGmmGlobalContext, CreateParams);
+    // ToDo: Only Vk is using this Create API directly. Derive the GmmLibCOntext from the ClientContext stored in
+    // ResInfo object.
+    Status = Create(*(reinterpret_cast<GMM_CLIENT_CONTEXT *>(pClientContext)->GetLibContext()), CreateParams);
 
     return Status;
 }
@@ -130,14 +126,14 @@ GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::CreateCustomRes(Context &G
 
     GMM_DPF_ENTER;
 
-    __GMM_ASSERTPTR(pGmmGlobalContext, GMM_ERROR);
-    pGmmLibContext = reinterpret_cast<uint64_t>(&GmmLibContext);
+    GET_GMM_CLIENT_TYPE(pClientContext, ClientType);
+    pGmmUmdLibContext = reinterpret_cast<uint64_t>(&GmmLibContext);
 
 
     if((CreateParams.Format > GMM_FORMAT_INVALID) &&
        (CreateParams.Format < GMM_RESOURCE_FORMATS))
     {
-        BitsPerPixel = pGmmGlobalContext->GetPlatformInfo().FormatTable[CreateParams.Format].Element.BitsPer;
+        BitsPerPixel = GetGmmLibContext()->GetPlatformInfo().FormatTable[CreateParams.Format].Element.BitsPer;
     }
     else
     {
@@ -146,8 +142,8 @@ GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::CreateCustomRes(Context &G
         goto ERROR_CASE;
     }
 
-    pPlatform    = GMM_OVERRIDE_PLATFORM_INFO(&Surf);
-    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf);
+    pPlatform    = GMM_OVERRIDE_PLATFORM_INFO(&Surf, GetGmmLibContext());
+    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf, GetGmmLibContext());
 
     Surf.Type                    = CreateParams.Type;
     Surf.Format                  = CreateParams.Format;
@@ -160,9 +156,10 @@ GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::CreateCustomRes(Context &G
     Surf.Alignment.BaseAlignment = CreateParams.BaseAlignment;
     Surf.MaxLod                  = 1;
     Surf.ArraySize               = 1;
+    Surf.CpTag                   = CreateParams.CpTag;
 
 #if(_DEBUG || _RELEASE_INTERNAL)
-    Surf.Platform = pGmmGlobalContext->GetPlatformInfo().Platform;
+    Surf.Platform = GetGmmLibContext()->GetPlatformInfo().Platform;
 #endif
     Surf.BitsPerPixel     = BitsPerPixel;
     Surf.Alignment.QPitch = (GMM_GLOBAL_GFX_SIZE_T)(Surf.Pitch * Surf.BaseHeight);
@@ -258,7 +255,9 @@ GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::Create(Context &GmmLibCont
 
     GMM_DPF_ENTER;
 
-    __GMM_ASSERTPTR(pGmmGlobalContext, GMM_ERROR);
+    GET_GMM_CLIENT_TYPE(pClientContext, ClientType);
+    pGmmUmdLibContext = reinterpret_cast<uint64_t>(&GmmLibContext);
+    __GMM_ASSERTPTR(pGmmUmdLibContext, GMM_ERROR);
 
     if(CreateParams.Flags.Info.ExistingSysMem &&
        (CreateParams.Flags.Info.TiledW ||
@@ -271,30 +270,14 @@ GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::Create(Context &GmmLibCont
         goto ERROR_CASE;
     }
 
-    pGmmLibContext = reinterpret_cast<uint64_t>(&GmmLibContext);
-
-#if(!defined(__GMM_KMD__) && !defined(GMM_UNIFIED_LIB))
-    // Client ULT does new on ResInfo without calling GmmInitGlobalContext. If they call create later, on the previously created
-    // ResInfo object set the clientContext for them, since clientContext wouldnt have been set
-    if(!pClientContext)
-    {
-        pClientContext = pGmmGlobalContext->pGmmGlobalClientContext;
-        GET_GMM_CLIENT_TYPE(pClientContext, ClientType);
-    }
-#endif
-
-#if defined(__GMM_KMD__)
-    ClientType = GMM_KMD_VISTA;
-#endif
-
     if(!CopyClientParams(CreateParams))
     {
         Status = GMM_INVALIDPARAM;
         goto ERROR_CASE;
     }
 
-    pPlatform    = GMM_OVERRIDE_PLATFORM_INFO(&Surf);
-    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf);
+    pPlatform    = GMM_OVERRIDE_PLATFORM_INFO(&Surf, GetGmmLibContext());
+    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf, GetGmmLibContext());
 
 #if defined(__GMM_KMD__) || !defined(_WIN32)
     if(!CreateParams.Flags.Info.ExistingSysMem)
@@ -317,16 +300,6 @@ GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::Create(Context &GmmLibCont
             goto ERROR_CASE;
         }
 
-        // Fill out the texture info for each plane if they require rediscription
-        if(Surf.Flags.Info.RedecribedPlanes)
-        {
-            if(false == RedescribePlanes())
-            {
-                GMM_ASSERTDPF(0, "Redescribe planes failed!");
-                goto ERROR_CASE;
-            }
-        }
-
         if(Surf.Flags.Gpu.UnifiedAuxSurface)
         {
             GMM_GFX_SIZE_T TotalSize;
@@ -349,6 +322,11 @@ GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::Create(Context &GmmLibCont
             if(Surf.Flags.Gpu.IndirectClearColor ||
                Surf.Flags.Gpu.ColorDiscard)
             {
+                if(GetGmmLibContext()->GetSkuTable().FtrFlatPhysCCS && AuxSurf.Type == RESOURCE_INVALID)
+                {
+                    //ie only AuxType is CCS, doesn't exist with FlatCCS, enable it for CC
+                    AuxSurf.Type = Surf.Type;
+                }
                 if(!Surf.Flags.Gpu.TiledResource)
                 {
                     AuxSurf.CCSize = PAGE_SIZE; // 128bit Float Value + 32bit RT Native Value + Padding.
@@ -360,6 +338,12 @@ GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::Create(Context &GmmLibCont
                     AuxSurf.Size += GMM_KBYTE(64);
                 }
             }
+	    
+	    if(Surf.Flags.Gpu.ProceduralTexture)
+            {
+                //Do not require main surface access either in GPUVA/physical space.
+                Surf.Size = 0;
+            }
 
             TotalSize = Surf.Size + AuxSurf.Size; //Not including AuxSecSurf size, multi-Aux surface isn't supported for displayables
             Alignment = GFX_ULONG_CAST(Surf.Pitch * pPlatform->TileInfo[Surf.TileMode].LogicalTileHeight);
@@ -475,7 +459,7 @@ GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::Create(Context &GmmLibCont
         }
     }
 
-    if(Is64KBPageSuitable() && pGmmGlobalContext->GetSkuTable().FtrLocalMemory)
+    if(Is64KBPageSuitable() && GetGmmLibContext()->GetSkuTable().FtrLocalMemory)
     {
         // BaseAlignment can be greater than 64KB and needs to be aligned to 64KB
         Surf.Alignment.BaseAlignment = GFX_MAX(GFX_ALIGN(Surf.Alignment.BaseAlignment, GMM_KBYTE(64)), GMM_KBYTE(64));
@@ -711,140 +695,6 @@ void GmmLib::GmmResourceInfoCommon::UpdateUnAlignedParams()
 
     __GMM_ASSERTPTR(((Surf.OffsetInfo.Plane.Y[GMM_PLANE_U] == YHeight) && (UmdUHeight == VHeight)), VOIDRETURN);
 }
-/////////////////////////////////////////////////////////////////////////////////////
-/// This function calculates number of planes required for the given input format
-/// and allocates texture info for the respective planes.
-///
-/// @return     ::bool
-/////////////////////////////////////////////////////////////////////////////////////
-bool GmmLib::GmmResourceInfoCommon::RedescribePlanes()
-{
-    const GMM_PLATFORM_INFO *pPlatform;
-    GMM_TEXTURE_CALC *       pTextureCalc = NULL;
-    GMM_STATUS               Status       = GMM_SUCCESS;
-    int                      MaxPlanes    = 1;
-
-    pPlatform    = GMM_OVERRIDE_PLATFORM_INFO(&Surf);
-    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf);
-
-    __GMM_ASSERT(Surf.Flags.Info.RedecribedPlanes);
-
-    GMM_TEXTURE_INFO *pYPlane = &PlaneSurf[GMM_PLANE_Y];
-    GMM_TEXTURE_INFO *pUPlane = &PlaneSurf[GMM_PLANE_U];
-    GMM_TEXTURE_INFO *pVPlane = &PlaneSurf[GMM_PLANE_V];
-
-    pYPlane->Type         = Surf.Type;
-    pYPlane->BaseWidth    = Surf.BaseWidth;
-    pYPlane->BaseHeight   = Surf.BaseHeight;
-    pYPlane->Depth        = Surf.Depth;
-    pYPlane->ArraySize    = Surf.ArraySize;
-    pYPlane->MSAA         = Surf.MSAA;
-    pYPlane->Flags        = Surf.Flags;
-    pYPlane->BitsPerPixel = Surf.BitsPerPixel;
-
-#if(_DEBUG || _RELEASE_INTERNAL)
-    pYPlane->Platform = Surf.Platform;
-#endif
-
-    pYPlane->Flags.Info.RedecribedPlanes = false;
-
-    *pUPlane = *pVPlane = *pYPlane;
-
-    if(GmmIsUVPacked(Surf.Format))
-    {
-        // UV packed resources must have two seperate
-        // tiling modes per plane, due to the packed
-        // UV plane having twice the bits per pixel
-        // as the Y plane.
-
-        if(Surf.BitsPerPixel == 8)
-        {
-            pYPlane->BitsPerPixel = 8;
-            pYPlane->Format       = GMM_FORMAT_R8_UINT;
-
-            pUPlane->BitsPerPixel = 16;
-            pUPlane->Format       = GMM_FORMAT_R16_UINT;
-        }
-        else if(Surf.BitsPerPixel == 16)
-        {
-            pYPlane->BitsPerPixel = 16;
-            pYPlane->Format       = GMM_FORMAT_R16_UINT;
-
-            pUPlane->BitsPerPixel = 32;
-            pUPlane->Format       = GMM_FORMAT_R32_UINT;
-        }
-        else
-        {
-            GMM_ASSERTDPF(0, "Unsupported format/pixel size combo!");
-            Status = GMM_INVALIDPARAM;
-            goto ERROR_CASE;
-        }
-
-        pUPlane->BaseHeight = GFX_CEIL_DIV(pYPlane->BaseHeight, 2);
-        pUPlane->BaseWidth  = GFX_CEIL_DIV(pYPlane->BaseWidth, 2);
-        MaxPlanes           = 2;
-    }
-    else
-    {
-        // Non-UV packed surfaces only require the plane descriptors
-        // have proper height and width for each plane
-        switch(Surf.Format)
-        {
-            case GMM_FORMAT_IMC1:
-            case GMM_FORMAT_IMC2:
-            case GMM_FORMAT_IMC3:
-            case GMM_FORMAT_IMC4:
-            case GMM_FORMAT_MFX_JPEG_YUV420:
-            {
-                pUPlane->BaseWidth = pVPlane->BaseWidth = GFX_CEIL_DIV(pYPlane->BaseWidth, 2);
-            }
-            case GMM_FORMAT_MFX_JPEG_YUV422V:
-            {
-                pUPlane->BaseHeight = pVPlane->BaseHeight = GFX_CEIL_DIV(pYPlane->BaseHeight, 2);
-                break;
-            }
-            case GMM_FORMAT_MFX_JPEG_YUV411R_TYPE:
-            {
-                pUPlane->BaseHeight = pVPlane->BaseHeight = GFX_CEIL_DIV(pYPlane->BaseHeight, 4);
-                break;
-            }
-            case GMM_FORMAT_MFX_JPEG_YUV411:
-            {
-                pUPlane->BaseWidth = pVPlane->BaseWidth = GFX_CEIL_DIV(pYPlane->BaseWidth, 4);
-                break;
-            }
-            case GMM_FORMAT_MFX_JPEG_YUV422H:
-            {
-                pUPlane->BaseWidth = pVPlane->BaseWidth = GFX_CEIL_DIV(pYPlane->BaseWidth, 2);
-                break;
-            }
-            default:
-            {
-                break;
-            }
-        }
-
-        pYPlane->Format = pUPlane->Format = pVPlane->Format =
-        (pYPlane->BitsPerPixel == 8) ? GMM_FORMAT_R8_UINT : GMM_FORMAT_R16_UINT;
-        MaxPlanes = 3;
-    }
-
-    for(int i = GMM_PLANE_Y; i <= MaxPlanes; i++) // all 2 or 3 planes
-    {
-        if((GMM_SUCCESS != pTextureCalc->AllocateTexture(&PlaneSurf[i])))
-        {
-            GMM_ASSERTDPF(false, "GmmTexAlloc failed!");
-            Status = GMM_ERROR;
-            goto ERROR_CASE;
-        }
-    }
-
-    Status = static_cast<GMM_STATUS>(false == ReAdjustPlaneProperties(false));
-
-ERROR_CASE:
-    return (Status == GMM_SUCCESS) ? true : false;
-}
-
 /////////////////////////////////////////////////////////////////////////////////////
 /// Returns downscaled width for fast clear of given subresource
 /// @param[in]  uint32_t : MipLevel
@@ -857,7 +707,7 @@ uint64_t GmmLib::GmmResourceInfoCommon::GetFastClearWidth(uint32_t MipLevel)
     uint32_t numSamples = GetNumSamples();
 
     GMM_TEXTURE_CALC *pTextureCalc;
-    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf);
+    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf, GetGmmLibContext());
 
     if(numSamples == 1)
     {
@@ -892,7 +742,7 @@ uint32_t GmmLib::GmmResourceInfoCommon::GetFastClearHeight(uint32_t MipLevel)
     uint32_t numSamples = GetNumSamples();
 
     GMM_TEXTURE_CALC *pTextureCalc;
-    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf);
+    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf, GetGmmLibContext());
 
     if(numSamples == 1)
     {
@@ -906,74 +756,6 @@ uint32_t GmmLib::GmmResourceInfoCommon::GetFastClearHeight(uint32_t MipLevel)
     return height;
 }
 
-
-/////////////////////////////////////////////////////////////////////////////////////
-/// This function readjustes Plane properties. Valid for MainSurf not for AuxSurf
-///
-/// @param[in]  bool: Whether Surf is Aux
-///
-/// @return     ::bool
-/////////////////////////////////////////////////////////////////////////////////////
-bool GmmLib::GmmResourceInfoCommon::ReAdjustPlaneProperties(bool IsAuxSurf)
-{
-    const GMM_PLATFORM_INFO *pPlatform     = GMM_OVERRIDE_PLATFORM_INFO(&Surf);
-    GMM_TEXTURE_INFO *       pTexInfo      = &Surf;
-    GMM_TEXTURE_INFO *       pPlaneTexInfo = PlaneSurf;
-
-    if(IsAuxSurf)
-    {
-        //AuxSurf isn't redescribed
-        return false;
-    }
-
-    if(GmmIsUVPacked(pTexInfo->Format))
-    {
-        pPlaneTexInfo[GMM_PLANE_V] = pPlaneTexInfo[GMM_PLANE_U];
-
-        // Need to adjust the returned surfaces and then copy
-        // the relivent data into the parent descriptor.
-        // UV plane is wider while Y plane is taller,
-        // so adjust pitch and sizes to fit accordingly
-        pTexInfo->Alignment        = pPlaneTexInfo[GMM_PLANE_U].Alignment;
-        pTexInfo->Alignment.VAlign = pPlaneTexInfo[GMM_PLANE_Y].Alignment.VAlign;
-
-        if(pPlaneTexInfo[GMM_PLANE_Y].Pitch != pPlaneTexInfo[GMM_PLANE_U].Pitch)
-        {
-            pPlaneTexInfo[GMM_PLANE_Y].Size = (pPlaneTexInfo[GMM_PLANE_Y].Size / pPlaneTexInfo[GMM_PLANE_Y].Pitch) * pPlaneTexInfo[GMM_PLANE_U].Pitch;
-            __GMM_ASSERT(GFX_IS_ALIGNED(pPlaneTexInfo[GMM_PLANE_Y].Size, pPlatform->TileInfo[pPlaneTexInfo[GMM_PLANE_Y].TileMode].LogicalSize));
-
-            if(pPlaneTexInfo[GMM_PLANE_Y].ArraySize > 1)
-            {
-                pPlaneTexInfo[GMM_PLANE_Y].OffsetInfo.Texture2DOffsetInfo.ArrayQPitchRender =
-                pPlaneTexInfo[GMM_PLANE_Y].OffsetInfo.Texture2DOffsetInfo.ArrayQPitchLock =
-                pPlaneTexInfo[GMM_PLANE_Y].Size / pPlaneTexInfo[GMM_PLANE_Y].ArraySize;
-            }
-
-            pTexInfo->Pitch = pPlaneTexInfo[GMM_PLANE_Y].Pitch = pPlaneTexInfo[GMM_PLANE_U].Pitch;
-        }
-
-        pTexInfo->OffsetInfo.Plane.ArrayQPitch =
-        pPlaneTexInfo[GMM_PLANE_Y].OffsetInfo.Texture2DOffsetInfo.ArrayQPitchRender +
-        pPlaneTexInfo[GMM_PLANE_U].OffsetInfo.Texture2DOffsetInfo.ArrayQPitchRender;
-
-        pTexInfo->Size = pPlaneTexInfo[GMM_PLANE_Y].Size + pPlaneTexInfo[GMM_PLANE_U].Size;
-
-        if(pTexInfo->Size > (GMM_GFX_SIZE_T)(pPlatform->SurfaceMaxSize))
-        {
-            GMM_ASSERTDPF(0, "Surface too large!");
-            return false;
-        }
-    }
-    else
-    {
-        // The parent resource should be the same size as all of the child planes
-        __GMM_ASSERT(pTexInfo->Size == (pPlaneTexInfo[GMM_PLANE_Y].Size +
-                                        pPlaneTexInfo[GMM_PLANE_U].Size + pPlaneTexInfo[GMM_PLANE_U].Size));
-    }
-
-    return true;
-}
-
 /////////////////////////////////////////////////////////////////////////////////////
 /// Returns the Platform info.  If Platform has been overriden by the clients, then
 /// it returns the overriden Platform Info struct.
@@ -982,16 +764,16 @@ bool GmmLib::GmmResourceInfoCommon::ReAdjustPlaneProperties(bool IsAuxSurf)
 const GMM_PLATFORM_INFO &GmmLib::GmmResourceInfoCommon::GetPlatformInfo()
 {
 #if(defined(__GMM_KMD__) && (_DEBUG || _RELEASE_INTERNAL))
-    if(GFX_GET_CURRENT_RENDERCORE(Surf.Platform) != GFX_GET_CURRENT_RENDERCORE(((Context *)pGmmLibContext)->GetPlatformInfo().Platform))
+    if(GFX_GET_CURRENT_RENDERCORE(Surf.Platform) != GFX_GET_CURRENT_RENDERCORE(((Context *)pGmmKmdLibContext)->GetPlatformInfo().Platform))
     {
-        return ((Context *)pGmmLibContext)->GetOverridePlatformInfo();
+        return ((Context *)pGmmKmdLibContext)->GetOverridePlatformInfo();
     }
     else
     {
-        return ((Context *)pGmmLibContext)->GetPlatformInfo();
+        return ((Context *)pGmmKmdLibContext)->GetPlatformInfo();
     }
 #else
-    return ((Context *)pGmmLibContext)->GetPlatformInfo();
+    return ((Context *)pGmmUmdLibContext)->GetPlatformInfo();
 #endif
 }
 
@@ -1011,7 +793,7 @@ uint32_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetPaddedWidth(uint32_t MipL
 
     __GMM_ASSERT(MipLevel <= Surf.MaxLod);
 
-    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf);
+    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf, GetGmmLibContext());
 
     // This shall be called for Depth and Separate Stencil main surface resource
     // This shall be called for the Aux surfaces (MCS, CCS and Hiz) too.
@@ -1096,7 +878,7 @@ uint32_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetPaddedHeight(uint32_t Mip
                  AuxSurf.Flags.Gpu.__MsaaTileMcs ||
                  AuxSurf.Flags.Gpu.CCS || AuxSurf.Flags.Gpu.__NonMsaaTileYCcs);
 
-    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf);
+    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf, GetGmmLibContext());
 
     MipHeight = pTextureCalc->GmmTexGetMipHeight(&Surf, MipLevel);
 
@@ -1186,7 +968,7 @@ uint32_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetQPitch()
     const GMM_PLATFORM_INFO *pPlatform;
     uint32_t                 QPitch;
 
-    pPlatform = GMM_OVERRIDE_PLATFORM_INFO(&Surf);
+    pPlatform = GMM_OVERRIDE_PLATFORM_INFO(&Surf, GetGmmLibContext());
 
     __GMM_ASSERT(GFX_GET_CURRENT_RENDERCORE(pPlatform->Platform) >= IGFX_GEN8_CORE);
     __GMM_ASSERT((Surf.Type != RESOURCE_3D) ||
@@ -1199,7 +981,7 @@ uint32_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetQPitch()
     // Stencil    ==> logical, i.e. not halved
 
     if((GFX_GET_CURRENT_RENDERCORE(pPlatform->Platform) >= IGFX_GEN9_CORE) &&
-       GmmIsCompressed(Surf.Format))
+       GmmIsCompressed(GetGmmLibContext(), Surf.Format))
     {
         QPitch = Surf.Alignment.QPitch / GetCompressionBlockHeight();
 
@@ -1231,6 +1013,12 @@ uint32_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetQPitch()
 /////////////////////////////////////////////////////////////////////////////////////
 GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetOffset(GMM_REQ_OFFSET_INFO &ReqInfo)
 {
+    GMM_TEXTURE_CALC *pTextureCalc;
+
+    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf, GetGmmLibContext());
+
+    __GMM_ASSERT((pTextureCalc != NULL));
+
     if(Surf.Flags.Info.RedecribedPlanes)
     {
         uint8_t RestoreReqStdLayout = ReqInfo.ReqStdLayout ? 1 : 0;
@@ -1239,13 +1027,14 @@ GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetOffset(GMM_REQ_OFFSET_I
         if(ReqInfo.ReqLock || ReqInfo.ReqRender)
         {
             ReqInfo.ReqStdLayout = 0;
-            GmmTexGetMipMapOffset(&Surf, &ReqInfo);
+            GmmTexGetMipMapOffset(&Surf, &ReqInfo, GetGmmLibContext());
             ReqInfo.ReqStdLayout = RestoreReqStdLayout;
         }
 
         if(ReqInfo.ReqStdLayout)
         {
             GMM_REQ_OFFSET_INFO TempReqInfo[GMM_MAX_PLANE] = {0};
+	    GMM_TEXTURE_INFO    TexInfo[GMM_MAX_PLANE];
             uint32_t            Plane, TotalPlanes = GmmLib::Utility::GmmGetNumPlanes(Surf.Format);
 
             // Caller must specify which plane they need the offset into if not
@@ -1262,12 +1051,16 @@ GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetOffset(GMM_REQ_OFFSET_I
             TempReqInfo[GMM_PLANE_Y].ReqLock = TempReqInfo[GMM_PLANE_Y].ReqRender = 0;
 
             TempReqInfo[GMM_PLANE_V] = TempReqInfo[GMM_PLANE_U] = TempReqInfo[GMM_PLANE_Y];
+            
+	    pTextureCalc->GetRedescribedPlaneParams(&Surf, GMM_PLANE_Y, &TexInfo[GMM_PLANE_Y]);
+            pTextureCalc->GetRedescribedPlaneParams(&Surf, GMM_PLANE_U, &TexInfo[GMM_PLANE_U]);
+            pTextureCalc->GetRedescribedPlaneParams(&Surf, GMM_PLANE_V, &TexInfo[GMM_PLANE_V]);
 
-            if(GMM_SUCCESS != GmmTexGetMipMapOffset(&PlaneSurf[GMM_PLANE_Y], &TempReqInfo[GMM_PLANE_Y]) ||
-               GMM_SUCCESS != GmmTexGetMipMapOffset(&PlaneSurf[GMM_PLANE_U], &TempReqInfo[GMM_PLANE_U]) ||
-               GMM_SUCCESS != GmmTexGetMipMapOffset(&PlaneSurf[GMM_PLANE_V], &TempReqInfo[GMM_PLANE_V]))
-            {
-                __GMM_ASSERT(0);
+            if(GMM_SUCCESS != GmmTexGetMipMapOffset(&TexInfo[GMM_PLANE_Y], &TempReqInfo[GMM_PLANE_Y], GetGmmLibContext()) ||
+               GMM_SUCCESS != GmmTexGetMipMapOffset(&TexInfo[GMM_PLANE_U], &TempReqInfo[GMM_PLANE_U], GetGmmLibContext()) ||
+               GMM_SUCCESS != GmmTexGetMipMapOffset(&TexInfo[GMM_PLANE_V], &TempReqInfo[GMM_PLANE_V], GetGmmLibContext()))
+	    {
+    		    __GMM_ASSERT(0);
                 return GMM_ERROR;
             }
 
@@ -1295,9 +1088,10 @@ GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetOffset(GMM_REQ_OFFSET_I
                 {
                     // Find the size of the previous planes and add it to the offset
                     TempReqInfo[Plane].StdLayout.Offset = -1;
-
-                    if(GMM_SUCCESS != GmmTexGetMipMapOffset(&PlaneSurf[Plane], &TempReqInfo[Plane]))
+                
+                    if(GMM_SUCCESS != GmmTexGetMipMapOffset(&TexInfo[Plane], &TempReqInfo[Plane], GetGmmLibContext()))
                     {
+                 
                         __GMM_ASSERT(0);
                         return GMM_ERROR;
                     }
@@ -1311,7 +1105,7 @@ GMM_STATUS GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetOffset(GMM_REQ_OFFSET_I
     }
     else
     {
-        return GmmTexGetMipMapOffset(&Surf, &ReqInfo);
+        return GmmTexGetMipMapOffset(&Surf, &ReqInfo, GetGmmLibContext());
     }
 }
 
@@ -1336,11 +1130,12 @@ uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::CpuBlt(GMM_RES_COPY_BLT *pBlt
     uint8_t                  Success = 1;
     GMM_TEXTURE_INFO *       pTexInfo;
     GMM_TEXTURE_CALC *       pTextureCalc;
+    GMM_TEXTURE_INFO         RedescribedPlaneInfo;
 
     __GMM_ASSERTPTR(pBlt, 0);
 
-    pPlatform    = GMM_OVERRIDE_PLATFORM_INFO(&Surf);
-    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf);
+    pPlatform    = GMM_OVERRIDE_PLATFORM_INFO(&Surf, GetGmmLibContext());
+    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf, GetGmmLibContext());
 
     __GMM_ASSERT(
     Surf.Type == RESOURCE_1D ||
@@ -1468,13 +1263,15 @@ uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::CpuBlt(GMM_RES_COPY_BLT *pBlt
 
         if(pBlt->Gpu.OffsetY < pTexInfo->OffsetInfo.Plane.Y[GMM_PLANE_U])
         {
+	    pTextureCalc->GetRedescribedPlaneParams(pTexInfo, GMM_PLANE_Y, &RedescribedPlaneInfo);
             // Y Plane
-            pTexInfo = &(PlaneSurf[GMM_PLANE_Y]);
+            pTexInfo = &RedescribedPlaneInfo;
         }
         else
         {
             // UV Plane
-            pTexInfo = &(PlaneSurf[GMM_PLANE_U]);
+	    pTextureCalc->GetRedescribedPlaneParams(pTexInfo, GMM_PLANE_U, &RedescribedPlaneInfo);
+            pTexInfo = &RedescribedPlaneInfo;
         }
     }
 
@@ -1728,16 +1525,19 @@ uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::CpuBlt(GMM_RES_COPY_BLT *pBlt
                 __GMM_ASSERT((SwizzledSurface.Pitch % 2) == 0);
                 SwizzledSurface.Pitch /= 2;
                 SwizzledSurface.Height *= 2;
-
-                __GMM_ASSERT((GetOffset.Render.XOffset % 2) == 0);
-                SwizzledSurface.OffsetX = GetOffset.Render.XOffset / 2 + __OffsetXBytes;
-                SwizzledSurface.OffsetY = GetOffset.Render.YOffset * 2 + __OffsetY;
             }
             else if(GMM_IS_4KB_TILE(pTexInfo->Flags) &&
                     !(pTexInfo->Flags.Info.TiledYf ||
                       GMM_IS_64KB_TILE(pTexInfo->Flags)))
             {
-                SwizzledSurface.pSwizzle = &INTEL_TILE_Y;
+                if(GetGmmLibContext()->GetSkuTable().FtrTileY)
+                 {
+                    SwizzledSurface.pSwizzle = &INTEL_TILE_Y;
+                }
+                else
+                {
+                    SwizzledSurface.pSwizzle = &INTEL_TILE_4;
+                }
             }
             else if(pTexInfo->Flags.Info.TiledX)
             {
@@ -1763,7 +1563,7 @@ uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::CpuBlt(GMM_RES_COPY_BLT *pBlt
                         CASE(Layout, Tile, msaa, xD, 128);    \
                     }
 
-                #define SWITCH_MSAA_INTEL(Layout, Tile, xD)     \
+                #define SWITCH_MSAA_TILE64(Layout, Tile, xD)     \
                 {\
                     switch(pTexInfo->MSAA.NumSamples)           \
                     {                                           \
@@ -1779,7 +1579,7 @@ uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::CpuBlt(GMM_RES_COPY_BLT *pBlt
                         case 4:                                 \
                         case 8:                                 \
                         case 16:                                \
-                            SWITCH_BPP(Layout, Tile, MSAA4_, xD);  \
+                            SWITCH_BPP(Layout, Tile, MSAA_, xD);  \
                             break;                              \
                     }\
                 }
@@ -1814,13 +1614,17 @@ uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::CpuBlt(GMM_RES_COPY_BLT *pBlt
                 {
                     if(pTexInfo->Flags.Info.TiledYf)
                     {
-                        SWITCH_BPP(INTEL, TILEYF, , 3D_);
+                        SWITCH_BPP(INTEL, TILE_YF, , 3D_);
                     }
                     else if(GMM_IS_64KB_TILE(pTexInfo->Flags))
                     {
-                        if(pGmmGlobalContext->GetSkuTable().FtrTileY)
+                        if(GetGmmLibContext()->GetSkuTable().FtrTileY)
                         {
-                            SWITCH_BPP(INTEL, TILEYS, , 3D_);
+                            SWITCH_BPP(INTEL, TILE_YS, , 3D_);
+                        }
+                        else
+                        {
+                            SWITCH_BPP(INTEL, TILE_64, , 3D_);
                         }
                     }
                 }
@@ -1828,13 +1632,17 @@ uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::CpuBlt(GMM_RES_COPY_BLT *pBlt
                 {
                     if(pTexInfo->Flags.Info.TiledYf)
                     {
-                        SWITCH_MSAA(INTEL, TILEYF, );
+                        SWITCH_MSAA(INTEL, TILE_YF, );
                     }
                     else if(GMM_IS_64KB_TILE(pTexInfo->Flags))
                     {
-                        if(pGmmGlobalContext->GetSkuTable().FtrTileY)
+                        if(GetGmmLibContext()->GetSkuTable().FtrTileY)
                         {
-                            SWITCH_MSAA(INTEL, TILEYS, );
+                            SWITCH_MSAA(INTEL, TILE_YS, );
+                        }
+                        else
+                        {
+                            SWITCH_MSAA_TILE64(INTEL, TILE_64, );
                         }
                     }
                 }
@@ -1873,12 +1681,14 @@ uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetMappingSpanDesc(GMM_GET_MA
     uint8_t                  WasFinalSpan = 0;
     GMM_TEXTURE_INFO *       pTexInfo;
     GMM_TEXTURE_CALC *       pTextureCalc;
+    GMM_TEXTURE_INFO         RedescribedPlaneInfo;
 
     __GMM_ASSERT(Surf.Flags.Info.StdSwizzle);
 
-    pPlatform    = GMM_OVERRIDE_PLATFORM_INFO(&Surf);
-    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf);
+    pPlatform    = GMM_OVERRIDE_PLATFORM_INFO(&Surf, GetGmmLibContext());
+    pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf, GetGmmLibContext());
 
+    __GMM_ASSERT(pTextureCalc != NULL);
     pTexInfo = &Surf;
 
     if(pMapping->Type == GMM_MAPPING_GEN9_YS_TO_STDSWIZZLE)
@@ -1937,7 +1747,9 @@ uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetMappingSpanDesc(GMM_GET_MA
                 pMapping->__NextSpan.VirtualOffset  = ReqInfo.Render.Offset64;
             }
 
-            pTexInfo = &PlaneSurf[pMapping->Scratch.Plane];
+	    pTextureCalc->GetRedescribedPlaneParams(pTexInfo, GMM_PLANE_Y, &RedescribedPlaneInfo);
+            pTexInfo = &RedescribedPlaneInfo;
+
         }
 
         // Initialization of Mapping Params...
@@ -2213,7 +2025,7 @@ uint32_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetPackedMipTailStartLod()
 {
     uint32_t NumPackedMips = 0, NumTilesForPackedMips = 0;
 
-    const GMM_PLATFORM_INFO *pPlatform = GMM_OVERRIDE_PLATFORM_INFO(&Surf);
+    const GMM_PLATFORM_INFO *pPlatform = GMM_OVERRIDE_PLATFORM_INFO(&Surf, GetGmmLibContext());
 
     GetTiledResourceMipPacking(&NumPackedMips,
                                &NumTilesForPackedMips);
@@ -2245,134 +2057,6 @@ bool GMM_STDCALL GmmLib::GmmResourceInfoCommon::IsMipRCCAligned(uint8_t &MisAlig
     return true;
 }
 
-/////////////////////////////////////////////////////////////////////////////////////
-/// Returns the resource's compressions block width
-/// @return    Compression block width
-/////////////////////////////////////////////////////////////////////////////////////
-uint32_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetCompressionBlockWidth()
-{
-    GMM_RESOURCE_FORMAT Format;
-    Format = Surf.Format;
-
-    __GMM_ASSERT((Format > GMM_FORMAT_INVALID) &&
-                 (Format < GMM_RESOURCE_FORMATS));
-
-    return pGmmGlobalContext->GetPlatformInfo().FormatTable[Format].Element.Width;
-}
-
-/////////////////////////////////////////////////////////////////////////////////////
-/// Returns the resource's compressions block height
-/// @return    Compression block width
-/////////////////////////////////////////////////////////////////////////////////////
-uint32_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetCompressionBlockHeight()
-{
-    GMM_RESOURCE_FORMAT Format;
-    Format = Surf.Format;
-
-    __GMM_ASSERT((Format > GMM_FORMAT_INVALID) &&
-                 (Format < GMM_RESOURCE_FORMATS));
-
-    return pGmmGlobalContext->GetPlatformInfo().FormatTable[Format].Element.Height;
-}
-
-/////////////////////////////////////////////////////////////////////////////////////
-/// Returns the resource's compressions block depth
-/// @return    Compression block width
-/////////////////////////////////////////////////////////////////////////////////////
-uint32_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetCompressionBlockDepth()
-{
-    GMM_RESOURCE_FORMAT Format;
-    Format = Surf.Format;
-
-    __GMM_ASSERT((Format > GMM_FORMAT_INVALID) &&
-                 (Format < GMM_RESOURCE_FORMATS));
-
-    return pGmmGlobalContext->GetPlatformInfo().FormatTable[Format].Element.Depth;
-}
-
-/////////////////////////////////////////////////////////////////////////////////////
-/// Returns whether resource uses LOD0-only or Full array spacing
-/// @return     1/0
-/////////////////////////////////////////////////////////////////////////////////////
-uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::IsArraySpacingSingleLod()
-{
-    __GMM_ASSERT(GFX_GET_CURRENT_RENDERCORE(pGmmGlobalContext->GetPlatformInfo().Platform) < IGFX_GEN8_CORE);
-    return Surf.Alignment.ArraySpacingSingleLod;
-}
-
-/////////////////////////////////////////////////////////////////////////////////////
-/// Returns whether resource is ASTC
-/// @return     1/0
-/////////////////////////////////////////////////////////////////////////////////////
-uint8_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::IsASTC()
-{
-    GMM_RESOURCE_FORMAT Format;
-    Format = Surf.Format;
-
-    return (Format > GMM_FORMAT_INVALID) &&
-           (Format < GMM_RESOURCE_FORMATS) &&
-           pGmmGlobalContext->GetPlatformInfo().FormatTable[Format].ASTC;
-}
-
-/////////////////////////////////////////////////////////////////////////////////////
-/// Returns MOCS associated with the resource
-/// @param[in]     MOCS
-/////////////////////////////////////////////////////////////////////////////////////
-MEMORY_OBJECT_CONTROL_STATE GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetMOCS()
-{
-    const GMM_CACHE_POLICY_ELEMENT *CachePolicy = pGmmGlobalContext->GetCachePolicyUsage();
-
-    __GMM_ASSERT(CachePolicy[GetCachePolicyUsage()].Initialized);
-
-    // Prevent wrong Usage for XAdapter resources. UMD does not call GetMemoryObject on shader resources but,
-    // when they add it someone could call it without knowing the restriction.
-    if(Surf.Flags.Info.XAdapter &&
-       GetCachePolicyUsage() != GMM_RESOURCE_USAGE_XADAPTER_SHARED_RESOURCE)
-    {
-        __GMM_ASSERT(false);
-    }
-
-    if((CachePolicy[GetCachePolicyUsage()].Override & CachePolicy[GetCachePolicyUsage()].IDCode) ||
-       (CachePolicy[GetCachePolicyUsage()].Override == ALWAYS_OVERRIDE))
-    {
-        return CachePolicy[GetCachePolicyUsage()].MemoryObjectOverride;
-    }
-
-    return CachePolicy[GetCachePolicyUsage()].MemoryObjectNoOverride;
-}
-
-/////////////////////////////////////////////////////////////////////////////////////
-/// Returns the surface state value for Standard Tiling Mode Extension
-/// @return     Standard Tiling Mode Extension
-/////////////////////////////////////////////////////////////////////////////////////
-uint32_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetStdTilingModeExtSurfaceState()
-{
-    const GMM_PLATFORM_INFO *pPlatform = GMM_OVERRIDE_PLATFORM_INFO(&Surf);
-    GMM_UNREFERENCED_LOCAL_VARIABLE(pPlatform); // Only used for debug
-    __GMM_ASSERT(GFX_GET_CURRENT_RENDERCORE(pPlatform->Platform) > IGFX_GEN10_CORE);
-
-    if(pGmmGlobalContext->GetSkuTable().FtrStandardMipTailFormat)
-    {
-        return 1;
-    }
-
-    return 0;
-}
-
-/////////////////////////////////////////////////////////////////////////////////////
-/// Returns the surface state value for Resource Format
-/// @return     Resource Format
-/////////////////////////////////////////////////////////////////////////////////////
-GMM_SURFACESTATE_FORMAT GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetResourceFormatSurfaceState()
-{
-    GMM_RESOURCE_FORMAT Format;
-
-    Format = Surf.Format;
-    __GMM_ASSERT((Format > GMM_FORMAT_INVALID) && (Format < GMM_RESOURCE_FORMATS));
-
-    return pGmmGlobalContext->GetPlatformInfo().FormatTable[Format].SurfaceStateFormat;
-}
-
 /////////////////////////////////////////////////////////////////////////////////////
 /// Return the logical width of mip level
 /// @param[in] MipLevel: Mip level for which the info is needed
@@ -2380,7 +2064,7 @@ GMM_SURFACESTATE_FORMAT GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetResourceFo
 /////////////////////////////////////////////////////////////////////////////////////
 GMM_GFX_SIZE_T GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetMipWidth(uint32_t MipLevel)
 {
-    GMM_TEXTURE_CALC *pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf);
+    GMM_TEXTURE_CALC *pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf, GetGmmLibContext());
     return pTextureCalc->GmmTexGetMipWidth(&Surf, MipLevel);
 }
 
@@ -2391,7 +2075,7 @@ GMM_GFX_SIZE_T GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetMipWidth(uint32_t M
 /////////////////////////////////////////////////////////////////////////////////////
 uint32_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetMipHeight(uint32_t MipLevel)
 {
-    GMM_TEXTURE_CALC *pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf);
+    GMM_TEXTURE_CALC *pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf, GetGmmLibContext());
     return pTextureCalc->GmmTexGetMipHeight(&Surf, MipLevel);
 }
 
@@ -2402,6 +2086,6 @@ uint32_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetMipHeight(uint32_t MipLev
 /////////////////////////////////////////////////////////////////////////////////////
 uint32_t GMM_STDCALL GmmLib::GmmResourceInfoCommon::GetMipDepth(uint32_t MipLevel)
 {
-    GMM_TEXTURE_CALC *pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf);
+    GMM_TEXTURE_CALC *pTextureCalc = GMM_OVERRIDE_TEXTURE_CALC(&Surf, GetGmmLibContext());
     return pTextureCalc->GmmTexGetMipDepth(&Surf, MipLevel);
 }
