@@ -25,14 +25,12 @@ OTHER DEALINGS IN THE SOFTWARE.
 #include "GmmConst.h"
 #include "../../../Platform/GmmPlatforms.h"
 
-#ifdef GMM_LIB_DLL
 #ifdef _WIN32
 #define GMM_MUTEX_HANDLE    HANDLE
 #else
 #include <pthread.h>
 #define GMM_MUTEX_HANDLE    pthread_mutex_t
 #endif
-#endif
 
 // Set packing alignment
 #pragma pack(push, 8)
@@ -81,7 +79,9 @@ namespace GmmLib
     class NON_PAGED_SECTION Context : public GmmMemAllocator
     {
     private:
-        static int32_t                   RefCount;
+#if(!defined(__GMM_KMD__) && !GMM_LIB_DLL_MA)
+    	static int32_t                   RefCount;
+#endif
         GMM_CLIENT                       ClientType;
         GMM_PLATFORM_INFO_CLASS*         pPlatformInfo;
 
@@ -112,7 +112,7 @@ namespace GmmLib
         // Padding Percentage limit on 64KB paged resource
         uint32_t               AllowedPaddingFor64KbPagesPercentage;
         uint64_t              InternalGpuVaMax;
-
+        uint32_t               AllowedPaddingFor64KBTileSurf;
 #ifdef GMM_LIB_DLL
         // Mutex Object used for synchronization of ProcessSingleton Context
         static GMM_MUTEX_HANDLE           SingletonContextSyncMutex;
@@ -123,6 +123,7 @@ namespace GmmLib
         Context();
         ~Context();
 
+#if(!defined(__GMM_KMD__) && (!GMM_LIB_DLL_MA))
         static int32_t IncrementRefCount()  // Returns the current RefCount and then increment it
         {
 #if defined(_WIN32)
@@ -155,7 +156,7 @@ namespace GmmLib
 
             return TargetValue;
         }
-
+#endif
         GMM_STATUS GMM_STDCALL InitContext(
                                     const PLATFORM& Platform,
                                     const SKU_FEATURE_TABLE* pSkuTable,
@@ -291,8 +292,25 @@ namespace GmmLib
         {
             return InternalGpuVaMax;
         }
+         /////////////////////////////////////////////////////////////////////////
+        /// Get padding  limit for 64k pages
+        /////////////////////////////////////////////////////////////////////////
+        uint32_t GetAllowedPaddingFor64KBTileSurf()
+        {
+            return (AllowedPaddingFor64KBTileSurf);
+        }
+
+        /////////////////////////////////////////////////////////////////////////
+        /// Set padding  limit for 64k pages
+        /////////////////////////////////////////////////////////////////////////
+
+        GMM_INLINE void SetAllowedPaddingFor64KBTileSurf(uint32_t Value)
+        {
+            AllowedPaddingFor64KBTileSurf = Value;
+        }
 
     #ifdef GMM_LIB_DLL
+        ADAPTER_BDF             sBdf;
         #ifdef _WIN32
             // ProcessHeapVA Singleton HeapObj
             GMM_HEAP            *pHeapObj;
@@ -537,18 +555,76 @@ namespace GmmLib
         }
     #endif
 
-
     #endif
+
+    GMM_CACHE_POLICY* GMM_STDCALL CreateCachePolicyCommon();
+    GMM_TEXTURE_CALC* GMM_STDCALL CreateTextureCalc(PLATFORM Platform, bool Override);
+    GMM_PLATFORM_INFO_CLASS *GMM_STDCALL CreatePlatformInfo(PLATFORM Platform, bool Override);
+    
     private: 
         void GMM_STDCALL OverrideSkuWa();
     };
+
+// Max number of Multi-Adapters allowed in the system
+// ToDO: Make this dynamic allocation instead of static Array
+#define MAX_NUM_ADAPTERS      3
+//===========================================================================
+// typedef:
+//      _GMM_ADAPTER_INFO_
+//
+// Description:
+//      Struct holds Adapter level information.
+//----------------------------------------------------------------------------
+typedef struct _GMM_ADAPTER_INFO_
+{
+    Context             *pGmmLibContext;                    // Gmm UMD Lib Context which is process Singleton
+    int32_t             RefCount;                           // Ref Count for the number of Gmm UMD Lib process Singleton Context created per Process
+    GMM_MUTEX_HANDLE    SyncMutex;                          // SyncMutex to protect access of Gmm UMD Lib process Singleton Context 
+    ADAPTER_BDF         sBdf;                               // Adpater's Bus, Device and Function info for which Gmm UMD Lib process Singleton Context is created
+}GMM_ADAPTER_INFO;
+    
+////////////////////////////////////////////////////////////////////////////////////
+/// Multi Adpater Context to hold data related to Multiple Adapters in the system
+/// Contains functions and members that are needed to support Multi-Adapter.
+///////////////////////////////////////////////////////////////////////////////////
+    class NON_PAGED_SECTION GmmMultiAdapterContext : public GmmMemAllocator
+    {
+    private:
+        GMM_ADAPTER_INFO                AdapterInfo[MAX_NUM_ADAPTERS];
+        GMM_MUTEX_HANDLE                MAContextSyncMutex;         // SyncMutex to protect access of GmmMultiAdpaterContext
+        uint32_t                        NumAdapters;
+	void*                           pCpuReserveBase;
+	uint64_t                        CpuReserveSize;
+
+    public:
+        //Constructors and destructors
+        GmmMultiAdapterContext();
+        ~GmmMultiAdapterContext();
+        /* Function prototypes */
+        /* Fucntions that update MultiAdapterContext members*/
+        uint32_t GMM_STDCALL            GetAdapterIndex(ADAPTER_BDF sBdf);
+        GMM_STATUS GMM_STDCALL          IntializeAdapterInfo(ADAPTER_BDF sBdf);
+        void GMM_STDCALL                ReleaseAdapterInfo(ADAPTER_BDF sBdf);
+        Context* GMM_STDCALL            GetAdapterLibContext(ADAPTER_BDF sBdf);
+        void GMM_STDCALL                SetAdapterLibContext(ADAPTER_BDF sBdf, Context* pGmmLibContext);
+        GMM_STATUS GMM_STDCALL          LockMAContextSyncMutex();
+        GMM_STATUS GMM_STDCALL          UnLockMAContextSyncMutex();
+        uint32_t GMM_STDCALL            GetNumAdapters();
+        /* Fucntions that update AdapterInfo*/
+        int32_t GMM_STDCALL             IncrementRefCount(ADAPTER_BDF sBdf);
+        int32_t GMM_STDCALL             DecrementRefCount(ADAPTER_BDF sBdf);
+        GMM_STATUS GMM_STDCALL          LockSingletonContextSyncMutex(ADAPTER_BDF sBdf);
+        GMM_STATUS GMM_STDCALL          UnlockSingletonContextSyncMutex(ADAPTER_BDF sBdf);
+    }; // GmmMultiAdapterContext
+
 } //namespace
 
-typedef GmmLib::Context GMM_GLOBAL_CONTEXT;
+typedef GmmLib::Context GMM_GLOBAL_CONTEXT, GMM_LIB_CONTEXT;
+typedef GmmLib::GmmMultiAdapterContext GMM_MA_LIB_CONTEXT;
 
 #else
 struct GmmLibContext;
-typedef struct GmmLibContext GMM_GLOBAL_CONTEXT;
+typedef struct GmmLibContext GMM_GLOBAL_CONTEXT, GMM_LIB_CONTEXT;
 #endif
 
 
@@ -591,14 +667,14 @@ extern "C" {
 }
 #endif /*__cplusplus*/
 
-    #define GMM_OVERRIDE_PLATFORM_INFO(pTexInfo)    (GmmGetPlatformInfo(pGmmGlobalContext))
-    #define GMM_OVERRIDE_TEXTURE_CALC(pTexInfo)     (GmmGetTextureCalc(pGmmGlobalContext))
+    #define GMM_OVERRIDE_PLATFORM_INFO(pTexInfo,pGmmLibContext)    (GmmGetPlatformInfo(pGmmLibContext))
+    #define GMM_OVERRIDE_TEXTURE_CALC(pTexInfo,pGmmLibContext)     (GmmGetTextureCalc(pGmmLibContext))
 
 #ifdef __GMM_KMD__
     #define GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(pTexInfo)    GMM_OVERRIDE_PLATFORM_INFO(pTexInfo)
     #define GMM_IS_PLANAR(Format)                            GmmIsPlanar(Format)
 #else
-    #define GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(pTexInfo)    (&pClientContext->GetPlatformInfo())
+    #define GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(pTexInfo,pGmmLibContext)    (&((GmmClientContext*)pClientContext)->GetPlatformInfo())
     #define GMM_IS_PLANAR(Format)                            (pClientContext->IsPlanar(Format))
 #endif
 
