@@ -64,19 +64,20 @@ namespace GmmLib
             GMM_TEXTURE_INFO                    Surf;                       ///< Contains info about the surface being created
             GMM_TEXTURE_INFO                    AuxSurf;                    ///< Contains info about the auxiliary surface if using Unified Auxiliary surfaces.
             GMM_TEXTURE_INFO                    AuxSecSurf;                 ///< For multi-Aux surfaces, contains info about the secondary auxiliary surface
-            GMM_TEXTURE_INFO                    PlaneSurf[GMM_MAX_PLANE];   ///< Contains info for each plane for tiled Ys/Yf planar resources
 
             uint32_t                            RotateInfo;
             GMM_EXISTING_SYS_MEM                ExistingSysMem;     ///< Info about resources initialized with existing system memory
             GMM_GFX_ADDRESS                     SvmAddress;         ///< Driver managed SVM address
 
-            uint64_t                            pGmmLibContext;     ///< Pointer to GmmLib context passed in during Create()
+            uint64_t                            pGmmUmdLibContext;     ///< Pointer to GmmLib context created in DLL passed in during Create()
+            uint64_t                            pGmmKmdLibContext;     ///< Pointer to GmmLib context created in KMD passed in during Create()
             uint64_t                            pPrivateData;       ///< Allows clients to attach any private data to GmmResourceInfo
 #ifdef __GMM_KMD__
-            void                               *pClientContext;    ///< void * in oreder to of same size for the ResInfo Object across KMD and UMD
+            void                               *pClientContext;    ///< void * in order to of same size for the ResInfo Object across KMD and UMD
 #else
             GmmClientContext                   *pClientContext;    ///< ClientContext of the client creating this Resource
 #endif
+            GMM_MULTI_TILE_ARCH                MultiTileArch;
 
         private:
             GMM_STATUS          ApplyExistingSysMemRestrictions();
@@ -86,19 +87,17 @@ namespace GmmLib
             GMM_VIRTUAL bool                IsPresentableformat();
             // Move GMM Restrictions to it's own class?
             virtual bool        CopyClientParams(GMM_RESCREATE_PARAMS &CreateParams);
-            GMM_VIRTUAL bool                RedescribePlanes();
-            GMM_VIRTUAL bool                ReAdjustPlaneProperties(bool IsAuxSurf);
             GMM_VIRTUAL const GMM_PLATFORM_INFO& GetPlatformInfo();
 
             /////////////////////////////////////////////////////////////////////////////////////
             /// Returns tile mode for SURFACE_STATE programming.
             /// @return     Tiled Mode
             /////////////////////////////////////////////////////////////////////////////////////
-            GMM_INLINE uint32_t GetTileModeSurfaceState(const GMM_TEXTURE_INFO *pTextureInfo) const
+            GMM_INLINE uint32_t GetTileModeSurfaceState(const GMM_TEXTURE_INFO *pTextureInfo)
             {
                 uint32_t TiledMode = 0;
 
-                if(GMM_IS_TILEY)
+                if(GMM_IS_TILEY(GetGmmLibContext()))
                 {
                     TiledMode =
                         pTextureInfo->Flags.Info.Linear ? 0 :
@@ -108,6 +107,16 @@ namespace GmmLib
 
                     __GMM_ASSERT((TiledMode != 3) || (pTextureInfo->Flags.Info.TiledY || pTextureInfo->Flags.Info.TiledYf || pTextureInfo->Flags.Info.TiledYs));
                 }
+                else
+                {
+                    TiledMode =
+                        (GMM_IS_4KB_TILE(pTextureInfo->Flags)) ? 3 :
+                            (GMM_IS_64KB_TILE(pTextureInfo->Flags)) ? 1 :
+                            pTextureInfo->Flags.Info.TiledX ? 2 :
+                            /* Linear */ 0;
+
+                    __GMM_ASSERT(TiledMode || pTextureInfo->Flags.Info.Linear);
+                }
 
                 return TiledMode;
             }
@@ -119,22 +128,15 @@ namespace GmmLib
                 Surf(),
                 AuxSurf(),
                 AuxSecSurf(),
-                PlaneSurf{},
                 RotateInfo(),
                 ExistingSysMem(),
                 SvmAddress(),
-                pGmmLibContext(),
+                pGmmUmdLibContext(),
+                pGmmKmdLibContext(),
                 pPrivateData(),
-                pClientContext()
+                pClientContext(),
+                MultiTileArch()
             {
-#if (!defined(__GMM_KMD__) && !defined(GMM_UNIFIED_LIB))
-                // For clients, who derive classes from GMM class and call their derived class constructors
-                if (pGmmGlobalContext) // Client ULT does new on ResInfo without calling GmmInitGlobalContext.
-                {
-                    pClientContext = pGmmGlobalContext->pGmmGlobalClientContext;
-                    GET_GMM_CLIENT_TYPE(pClientContext, ClientType);
-                }
-#endif
             }
 
 #ifndef __GMM_KMD__
@@ -143,13 +145,14 @@ namespace GmmLib
                 Surf(),
                 AuxSurf(),
                 AuxSecSurf(),
-                PlaneSurf{},
                 RotateInfo(),
                 ExistingSysMem(),
                 SvmAddress(),
-                pGmmLibContext(),
+                pGmmUmdLibContext(),
+                pGmmKmdLibContext(),
                 pPrivateData(),
-                pClientContext()
+                pClientContext(),
+                MultiTileArch()
             {
                 pClientContext = pClientContextIn;
             }
@@ -165,7 +168,7 @@ namespace GmmLib
                 ExistingSysMem      = rhs.ExistingSysMem;
                 SvmAddress          = rhs.SvmAddress;
                 pPrivateData        = rhs.pPrivateData;
-                pGmmLibContext      = rhs.pGmmLibContext;
+                MultiTileArch       = rhs.MultiTileArch;
 
                 return *this;
             }
@@ -198,14 +201,6 @@ namespace GmmLib
             GMM_VIRTUAL bool                    GMM_STDCALL IsMipRCCAligned(uint8_t &MisAlignedLod);
             GMM_VIRTUAL uint8_t                 GMM_STDCALL GetDisplayFastClearSupport();
             GMM_VIRTUAL uint8_t                 GMM_STDCALL GetDisplayCompressionSupport();
-            GMM_VIRTUAL uint32_t                GMM_STDCALL GetCompressionBlockWidth();
-            GMM_VIRTUAL uint32_t                GMM_STDCALL GetCompressionBlockHeight();
-            GMM_VIRTUAL uint32_t                GMM_STDCALL GetCompressionBlockDepth();
-            GMM_VIRTUAL uint8_t                 GMM_STDCALL IsArraySpacingSingleLod();
-            GMM_VIRTUAL uint8_t                 GMM_STDCALL IsASTC();
-            GMM_VIRTUAL MEMORY_OBJECT_CONTROL_STATE GMM_STDCALL GetMOCS();
-            GMM_VIRTUAL uint32_t                GMM_STDCALL GetStdTilingModeExtSurfaceState();
-            GMM_VIRTUAL GMM_SURFACESTATE_FORMAT GMM_STDCALL GetResourceFormatSurfaceState();
             GMM_VIRTUAL GMM_GFX_SIZE_T          GMM_STDCALL GetMipWidth(uint32_t MipLevel);
             GMM_VIRTUAL uint32_t                GMM_STDCALL GetMipHeight(uint32_t MipLevel);
             GMM_VIRTUAL uint32_t                GMM_STDCALL GetMipDepth(uint32_t MipLevel);
@@ -236,6 +231,50 @@ namespace GmmLib
             }
 #endif
 
+
+
+            /////////////////////////////////////////////////////////////////////////////////////
+            /// This function Sets GmmLibContext in GmmResInfo.
+            /// GMMResInfo gets passed as private data from UMDs to KMD during 
+            //  GMMCreateAllocation and GmmOpenAllocation
+            /// Member functions of ResInfo class need to access the LibContext object. But since
+            /// the LibContext object present in ResInfo is created by UMDs in UMD space,
+            /// and this UMD space object cant be used in KMD space, 
+            /// we need this API to set the KMD LibContext in ResInfo object.
+            /// @return ::void
+            /////////////////////////////////////////////////////////////////////////////////////
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED void SetGmmLibContext(void *pLibContext)
+            {
+#if(defined(__GMM_KMD__))
+                pGmmKmdLibContext = reinterpret_cast<uint64_t>(pLibContext);
+#else
+                pGmmUmdLibContext = reinterpret_cast<uint64_t>(pLibContext);
+#endif
+            }
+
+            /////////////////////////////////////////////////////////////////////////////////////
+            /// Returns either UMD or KMD GMMLibContext that needs to be used when the ResInfo
+            /// Member functions are executed at KMD or UMD level
+            /// @return ::Context
+            /////////////////////////////////////////////////////////////////////////////////////
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED Context *GetGmmLibContext()
+            {
+#if(defined(__GMM_KMD__))
+                return ((Context *)pGmmKmdLibContext);
+#else
+                return ((Context *)pGmmUmdLibContext);
+#endif
+            }
+
+            /////////////////////////////////////////////////////////////////////////////////////
+            /// Returns GMM_CLIENT Type that has created this resource
+            /// @return ::GMM_CLIENT
+            /////////////////////////////////////////////////////////////////////////////////////
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED void SetClientType(GMM_CLIENT Client)
+            {
+                ClientType = Client;
+            }
+            
             /////////////////////////////////////////////////////////////////////////////////////
             /// Returns GMM_CLIENT Type that has created this resource
             /// @return ::GMM_CLIENT
@@ -247,7 +286,7 @@ namespace GmmLib
 
             /////////////////////////////////////////////////////////////////////////////////////
             /// Returns the system memory pointer. It selectively returns either the natural
-            /// pointer or a value appriopriately page aligned for D3DDI_ALLOCATIONINFO,
+            /// pointer or a value appropriately page aligned for D3DDI_ALLOCATIONINFO,
             /// depending on what the caller request.
             /// @param[in]      IsD3DDdiAllocation: Specifies where allocation was made by a D3D client
             /// @return         Pointer to system memory. NULL if not available.
@@ -380,9 +419,9 @@ namespace GmmLib
             {
                 const GMM_PLATFORM_INFO   *pPlatform;
 
-                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf);
-
-                if (Surf.Flags.Gpu.UnifiedAuxSurface)
+                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf, GetGmmLibContext());
+                
+		if (Surf.Flags.Gpu.UnifiedAuxSurface)
                 {
                     if (GMM_IS_PLANAR(Surf.Format))
                     {
@@ -416,8 +455,9 @@ namespace GmmLib
                 __GMM_ASSERT(GMM_IS_PLANAR(Surf.Format));
                 GMM_UNREFERENCED_LOCAL_VARIABLE(Plane);
 
-                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf);
-                __GMM_ASSERT(GFX_GET_CURRENT_RENDERCORE(pPlatform->Platform) >= IGFX_GEN8_CORE);
+                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf, GetGmmLibContext());
+	       
+	        __GMM_ASSERT(GFX_GET_CURRENT_RENDERCORE(pPlatform->Platform) >= IGFX_GEN8_CORE);
 
                 QPitch = static_cast<uint32_t>(Surf.OffsetInfo.Plane.ArrayQPitch / Surf.Pitch);
 
@@ -458,11 +498,11 @@ namespace GmmLib
                 TileMode = Surf.TileMode;
                 __GMM_ASSERT(TileMode < GMM_TILE_MODES);
 
-                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf);
+                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf, GetGmmLibContext());
                 if (pPlatform->TileInfo[TileMode].LogicalTileWidth != 0)
                 {
                     // In case of Depth/Stencil buffer MSAA TileYs surface, the LogicalTileWidth/Height is smaller than non-MSAA ones
-                    // Thus introducting the below variable to get the right PitchInTiles
+                    // Thus introducing the below variable to get the right PitchInTiles
                     uint32_t MSAASpecialFactorForDepthAndStencil = 1;
 
                     if ((Surf.Flags.Gpu.Depth || Surf.Flags.Gpu.SeparateStencil) &&
@@ -516,7 +556,7 @@ namespace GmmLib
 
                 __GMM_ASSERT(!AuxSurf.Flags.Info.Linear);
 
-                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&AuxSurf);
+                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&AuxSurf, GetGmmLibContext());
 
                 if (Surf.Flags.Gpu.UnifiedAuxSurface)
                 {
@@ -584,6 +624,14 @@ namespace GmmLib
                 {
                     return GMM_TILED_Y;
                 }
+                else if (Surf.Flags.Info.Tile4)
+                {
+                    return GMM_TILED_4;
+                }
+                else if (Surf.Flags.Info.Tile64)
+                {
+                    return GMM_TILED_64;
+                }
 
                 return GMM_NOT_TILED;
             }
@@ -736,7 +784,8 @@ namespace GmmLib
                 __GMM_ASSERT(ArrayIndex < Surf.ArraySize);
                 __GMM_ASSERT(GMM_IS_PLANAR(Surf.Format));
 
-                if (Surf.Flags.Gpu.UnifiedAuxSurface)
+                if (Surf.Flags.Gpu.UnifiedAuxSurface  &&
+                    !(GetGmmLibContext()->GetSkuTable().FtrFlatPhysCCS))
                 {
                     if (GmmAuxType == GMM_AUX_Y_CCS)
                     {
@@ -778,7 +827,8 @@ namespace GmmLib
             {
                 const __GMM_PLATFORM_RESOURCE   *pPlatformResource;
                 uint32_t HAlign;
-                pPlatformResource = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf);
+
+                pPlatformResource = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf, GetGmmLibContext());
 
                 if ((GFX_GET_CURRENT_RENDERCORE(pPlatformResource->Platform) >= IGFX_GEN9_CORE) &&
                     !(Surf.Flags.Info.TiledYf || GMM_IS_64KB_TILE(Surf.Flags)))
@@ -801,7 +851,7 @@ namespace GmmLib
             {
                 const __GMM_PLATFORM_RESOURCE   *pPlatformResource;
                 uint32_t VAlign;
-                pPlatformResource = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf);
+                pPlatformResource = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf, GetGmmLibContext());
 
                 if ((GFX_GET_CURRENT_RENDERCORE(pPlatformResource->Platform) >= IGFX_GEN9_CORE) &&
                     !(GetResFlags().Info.TiledYf || GMM_IS_64KB_TILE(GetResFlags())))
@@ -976,7 +1026,8 @@ namespace GmmLib
             /////////////////////////////////////////////////////////////////////////////////////
             GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED GMM_GFX_SIZE_T  GMM_STDCALL GetSizeSurface()
             {
-                    return (Surf.Size + AuxSurf.Size + AuxSecSurf.Size);
+                GMM_OVERRIDE_SIZE_64KB_ALLOC(GetGmmLibContext());
+                return (Surf.Size + AuxSurf.Size + AuxSecSurf.Size);
             }
 
             /////////////////////////////////////////////////////////////////////////////////////
@@ -1001,7 +1052,7 @@ namespace GmmLib
             /////////////////////////////////////////////////////////////////////////////////////
             GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED uint32_t  GMM_STDCALL GetMaxGpuVirtualAddressBits()
             {
-                const GMM_PLATFORM_INFO *pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf);
+                const GMM_PLATFORM_INFO *pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf, GetGmmLibContext());
                 __GMM_ASSERTPTR(pPlatform, 0);
 
                 return pPlatform->MaxGpuVirtualAddressBitsPerResource;
@@ -1016,7 +1067,7 @@ namespace GmmLib
             {
                 GMM_GFX_SIZE_T Offset = 0;
                 const GMM_PLATFORM_INFO *pPlatform;
-                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf);
+                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf, GetGmmLibContext());
                 if (Surf.Flags.Gpu.UnifiedAuxSurface)
                 {
                     if ((GmmAuxType == GMM_AUX_CCS) || (GmmAuxType == GMM_AUX_SURF) || (GmmAuxType == GMM_AUX_Y_CCS)
@@ -1076,6 +1127,13 @@ namespace GmmLib
                     Offset = 0;
                 }
 
+                if((GetGmmLibContext()->GetSkuTable().FtrFlatPhysCCS) && !Surf.Flags.Gpu.ProceduralTexture &&
+                    (GmmAuxType == GMM_AUX_CCS || GmmAuxType == GMM_AUX_ZCS ||
+                    GmmAuxType == GMM_AUX_Y_CCS || GmmAuxType == GMM_AUX_UV_CCS))
+                {
+                    Offset = 0;
+                }
+
                 return Offset;
             }
 
@@ -1092,6 +1150,11 @@ namespace GmmLib
                 }
                 else if (GmmAuxType == GMM_AUX_CCS || GmmAuxType == GMM_AUX_HIZ || GmmAuxType == GMM_AUX_MCS)
                 {
+                    if(GmmAuxType == GMM_AUX_CCS &&
+                       GetGmmLibContext()->GetSkuTable().FtrFlatPhysCCS && !Surf.Flags.Gpu.ProceduralTexture)
+                    {
+                        return 0;
+                    }
                     if (GmmAuxType == GMM_AUX_CCS && AuxSecSurf.Type != RESOURCE_INVALID &&
                         (Surf.Flags.Gpu.CCS && (Surf.MSAA.NumSamples > 1 ||
                             Surf.Flags.Gpu.Depth)))
@@ -1305,17 +1368,17 @@ namespace GmmLib
                 uint32_t               HAlign = 0;
                 const GMM_PLATFORM_INFO   *pPlatform;
 
-                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf);
+                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf, GetGmmLibContext());
 
                 if (GFX_GET_CURRENT_RENDERCORE(pPlatform->Platform) >= IGFX_GEN8_CORE)
                 {
                     if (GetResFlags().Info.TiledYf || GMM_IS_64KB_TILE(GetResFlags()))
                     {
-                        HAlign = 1; //Ignored, but we'll retrun valid encoding nonetheless.
+                        HAlign = 1; // Ignored, but we'll retrun valid encoding nonetheless.
                     }
                     else
                     {
-                        if(GMM_IS_TILEY)
+                        if(GMM_IS_TILEY(GetGmmLibContext()))
                         {
                             switch (GetHAlign())
                             {
@@ -1325,6 +1388,24 @@ namespace GmmLib
                                 default: HAlign = 1; // TODO(Benign): Change back to 0 + assert after packed YUV handling corrected.
                             }
                         }
+                        else
+                        {
+                            uint32_t Align = GetHAlign() * (GetBitsPerPixel() >> 3);
+
+                            if (Surf.BitsPerPixel == 24 || Surf.BitsPerPixel == 48 || Surf.BitsPerPixel == 96)
+                            {
+                                Align = GetHAlign();
+                            }
+
+                            switch (Align)
+                            {
+                                case  16:  HAlign = 0; break;
+                                case  32:  HAlign = 1; break;
+                                case  64:  HAlign = 2; break;
+                                case 128:  HAlign = 3; break;
+                                default:   HAlign = 0; __GMM_ASSERT(0);
+                            }
+                        }
                     }
                 }
                 else
@@ -1349,13 +1430,13 @@ namespace GmmLib
                 uint32_t               VAlign;
                 const GMM_PLATFORM_INFO   *pPlatform;
 
-                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf);
+                pPlatform = (GMM_PLATFORM_INFO *)GMM_OVERRIDE_EXPORTED_PLATFORM_INFO(&Surf, GetGmmLibContext());
 
                 if (GFX_GET_CURRENT_RENDERCORE(pPlatform->Platform) >= IGFX_GEN8_CORE)
                 {
                     if (GetResFlags().Info.TiledYf || GMM_IS_64KB_TILE(GetResFlags()))
                     {
-                        VAlign = 1; // Ignored , but we'll return valid encoding nonetheless.
+                        VAlign = 1; // Ignored, but we'll return valid encoding nonetheless.
                     }
                     else
                     {
@@ -1389,9 +1470,9 @@ namespace GmmLib
             GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED uint32_t GMM_STDCALL GetTileModeSurfaceState()
             {
                 return GetTileModeSurfaceState(&Surf);
-			}
+	    }
 			
-			/////////////////////////////////////////////////////////////////////////////////////
+	    /////////////////////////////////////////////////////////////////////////////////////
             /// Returns tile mode for AUX SURFACE_STATE programming.
             /// @return     Tiled Mode
             /////////////////////////////////////////////////////////////////////////////////////
@@ -1408,7 +1489,7 @@ namespace GmmLib
             {
                 uint32_t   TiledResourceMode = 0;
 
-                if(GMM_IS_TILEY)
+                if(GMM_IS_TILEY(GetGmmLibContext()))
                 {
                     if (Surf.Flags.Info.TiledYf)
                     {
@@ -1609,8 +1690,12 @@ namespace GmmLib
             /////////////////////////////////////////////////////////////////////////////////////
             GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED void GMM_STDCALL OverrideGmmLibContext(Context *pNewGmmLibContext)
             {
-                this->pGmmLibContext = reinterpret_cast<uint64_t>(pNewGmmLibContext);
-            }
+#if(defined(__GMM_KMD__))
+                this->pGmmKmdLibContext = reinterpret_cast<uint64_t>(pNewGmmLibContext);
+#else
+                this->pGmmUmdLibContext = reinterpret_cast<uint64_t>(pNewGmmLibContext);
+#endif
+             }
 
             /////////////////////////////////////////////////////////////////////////////////////
             /// Overrides the X offset of planar surface
@@ -1637,6 +1722,198 @@ namespace GmmLib
             GMM_VIRTUAL GMM_STATUS              GMM_STDCALL CreateCustomRes(Context& GmmLibContext, GMM_RESCREATE_CUSTOM_PARAMS& CreateParams);
             protected:
                 GMM_VIRTUAL void UpdateUnAlignedParams();
+            public:
+            /////////////////////////////////////////////////////////////////////////////////////
+            /// Returns the resource's compressions block width
+            /// @return    Compression block width
+            /////////////////////////////////////////////////////////////////////////////////////
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED uint32_t GMM_STDCALL GetCompressionBlockWidth()
+            {
+                GMM_RESOURCE_FORMAT Format;
+                Format = Surf.Format;
+
+                __GMM_ASSERT((Format > GMM_FORMAT_INVALID) &&
+                             (Format < GMM_RESOURCE_FORMATS));
+
+                return GetGmmLibContext()->GetPlatformInfo().FormatTable[Format].Element.Width;
+            }
+
+            /////////////////////////////////////////////////////////////////////////////////////
+            /// Returns the resource's compressions block height
+            /// @return    Compression block width
+            /////////////////////////////////////////////////////////////////////////////////////
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED uint32_t GMM_STDCALL GetCompressionBlockHeight()
+            {
+                GMM_RESOURCE_FORMAT Format;
+                Format = Surf.Format;
+
+                __GMM_ASSERT((Format > GMM_FORMAT_INVALID) &&
+                             (Format < GMM_RESOURCE_FORMATS));
+
+                return GetGmmLibContext()->GetPlatformInfo().FormatTable[Format].Element.Height;
+            }
+
+            /////////////////////////////////////////////////////////////////////////////////////
+            /// Returns the resource's compressions block depth
+            /// @return    Compression block width
+            /////////////////////////////////////////////////////////////////////////////////////
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED uint32_t GMM_STDCALL GetCompressionBlockDepth()
+            {
+                GMM_RESOURCE_FORMAT Format;
+                Format = Surf.Format;
+
+                __GMM_ASSERT((Format > GMM_FORMAT_INVALID) &&
+                             (Format < GMM_RESOURCE_FORMATS));
+
+                return GetGmmLibContext()->GetPlatformInfo().FormatTable[Format].Element.Depth;
+            }
+
+            /////////////////////////////////////////////////////////////////////////////////////
+            /// Returns whether resource uses LOD0-only or Full array spacing
+            /// @return     1/0
+            /////////////////////////////////////////////////////////////////////////////////////
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED uint8_t GMM_STDCALL IsArraySpacingSingleLod()
+            {
+                __GMM_ASSERT(GFX_GET_CURRENT_RENDERCORE(GetGmmLibContext()->GetPlatformInfo().Platform) < IGFX_GEN8_CORE);
+                return Surf.Alignment.ArraySpacingSingleLod;
+            }
+
+            /////////////////////////////////////////////////////////////////////////////////////
+            /// Returns whether resource is ASTC
+            /// @return     1/0
+            /////////////////////////////////////////////////////////////////////////////////////
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED uint8_t GMM_STDCALL IsASTC()
+            {
+                GMM_RESOURCE_FORMAT Format;
+                Format = Surf.Format;
+
+                return (Format > GMM_FORMAT_INVALID) &&
+                       (Format < GMM_RESOURCE_FORMATS) &&
+                       GetGmmLibContext()->GetPlatformInfo().FormatTable[Format].ASTC;
+            }
+
+            /////////////////////////////////////////////////////////////////////////////////////
+            /// Returns MOCS associated with the resource
+            /// @param[in]     MOCS
+            /////////////////////////////////////////////////////////////////////////////////////
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED MEMORY_OBJECT_CONTROL_STATE GMM_STDCALL GetMOCS()
+            {
+                const GMM_CACHE_POLICY_ELEMENT *CachePolicy = GetGmmLibContext()->GetCachePolicyUsage();
+
+                __GMM_ASSERT(CachePolicy[GetCachePolicyUsage()].Initialized);
+
+                // Prevent wrong Usage for XAdapter resources. UMD does not call GetMemoryObject on shader resources but,
+                // when they add it someone could call it without knowing the restriction.
+                if(Surf.Flags.Info.XAdapter &&
+                   GetCachePolicyUsage() != GMM_RESOURCE_USAGE_XADAPTER_SHARED_RESOURCE)
+                {
+                    __GMM_ASSERT(false);
+                }
+
+                if((CachePolicy[GetCachePolicyUsage()].Override & CachePolicy[GetCachePolicyUsage()].IDCode) ||
+                   (CachePolicy[GetCachePolicyUsage()].Override == ALWAYS_OVERRIDE))
+                {
+                    return CachePolicy[GetCachePolicyUsage()].MemoryObjectOverride;
+                }
+
+                return CachePolicy[GetCachePolicyUsage()].MemoryObjectNoOverride;
+            }
+
+            /////////////////////////////////////////////////////////////////////////////////////
+            /// Returns the surface state value for Standard Tiling Mode Extension
+            /// @return     Standard Tiling Mode Extension
+            /////////////////////////////////////////////////////////////////////////////////////
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED uint32_t GMM_STDCALL GetStdTilingModeExtSurfaceState()
+            {
+                __GMM_ASSERT(GFX_GET_CURRENT_RENDERCORE(GetGmmLibContext()->GetPlatformInfo().Platform) > IGFX_GEN10_CORE);
+
+                if(GetGmmLibContext()->GetSkuTable().FtrStandardMipTailFormat)
+                {
+                    return 1;
+                }
+
+                return 0;
+            }
+
+            /////////////////////////////////////////////////////////////////////////////////////
+            /// Returns the surface state value for Resource Format
+            /// @return     Resource Format
+            /////////////////////////////////////////////////////////////////////////////////////
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED GMM_SURFACESTATE_FORMAT GMM_STDCALL GetResourceFormatSurfaceState()
+            {
+                GMM_RESOURCE_FORMAT Format;
+
+                Format = Surf.Format;
+                __GMM_ASSERT((Format > GMM_FORMAT_INVALID) && (Format < GMM_RESOURCE_FORMATS));
+
+                return GetGmmLibContext()->GetPlatformInfo().FormatTable[Format].SurfaceStateFormat;
+            }
+
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED const GMM_MULTI_TILE_ARCH& GetMultiTileArch()
+            {
+                return MultiTileArch;
+            }
+
+            /////////////////////////////////////////////////////////////////////////////////////
+            /// Returns the Flat Phys CCS Size for the resource
+            /// @return     CCS size in bytes
+            /////////////////////////////////////////////////////////////////////////////////////
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED GMM_GFX_SIZE_T GMM_STDCALL GetFlatPhysCcsSize()
+            {
+                if((GetGmmLibContext()->GetSkuTable().FtrFlatPhysCCS) &&
+                    !(Surf.Flags.Info.AllowVirtualPadding ||
+                        Surf.Flags.Info.ExistingSysMem ||
+                        Surf.Flags.Info.NonLocalOnly))
+                {
+                    return GFX_CEIL_DIV(Surf.Size, 256);
+                }
+                return 0;
+            }
+			/////////////////////////////////////////////////////////////////////////////////////
+            /// Returns Tiled mode for DEPTH_BUFFER_STATE/STENCIL_BUFFER_STATE/ HIER_DEPTH_BUFFER programming.
+            /// HIZ is always 4kb tiling, XeHP+ TileMode for HIZ is Tile4 and main surface can be
+            /// Tile64 , GMM_AUX_INVALID, will return data for main depth/stencil resource ,
+            /// GMM_AUX_HiZ returns data for HIZ resource
+            /// @return     Tiled Resource Mode(PreGen12) / Tiled Mode(Gen12+)
+            /////////////////////////////////////////////////////////////////////////////////////
+            GMM_INLINE_VIRTUAL GMM_INLINE_EXPORTED uint32_t GMM_STDCALL GetTiledModeDepthStencilState( GMM_UNIFIED_AUX_TYPE  AuxType = GMM_AUX_INVALID)
+            {
+                uint32_t TiledMode = 0;
+
+                if(GMM_IS_TILEY(GetGmmLibContext()))
+                {
+                    TiledMode =
+				Surf.Flags.Info.TiledYf ? 1 :
+				Surf.Flags.Info.TiledYs ? 2 :
+			        /*TILE_NONE*/		  0;
+                }
+                else
+                {
+                    //1 and 3 are only valid value , 0 and 2 are reserved for XeHP+
+                    if( (AuxType == GMM_AUX_HIZ) && AuxSurf.Flags.Gpu.HiZ )
+                    {
+                        TiledMode =
+                            AuxSurf.Flags.Info.Tile4    ? 3 :
+                            AuxSurf.Flags.Info.Tile64   ? 1 :
+                            /* Default */                 0;
+
+                        __GMM_ASSERT(TiledMode == 3);
+                    }
+                    else
+                    {
+                        TiledMode =
+                            Surf.Flags.Info.Tile4   ? 3 :
+                            Surf.Flags.Info.Tile64  ? 1 :
+                            /* Default */	      0;
+
+                        __GMM_ASSERT( TiledMode );
+
+                    }
+
+                }
+
+                return TiledMode;
+            }
 
     };
 
